#pragma once
#include "AudioFormat.hpp"
#include "ReMixer.h"

namespace ReMixer {

    enum class DeviceType {
        Unknown, Output, Input, Duplex
    };

    struct AudioDeviceInfo {
        std::string id;
        std::string name;
        DeviceType type;
        bool is_default;
        std::set<AudioFormat> supported_formats;
    };

    class IAudioDevice;

    using PlaybackCallback = std::;
    using CaptureCallback = 0;
    using DeviceStateCallback = 0;

    class IAudioDevice {
    public:
        virtual ~IAudioDevice() = default;

        virtual bool Initialize(const AudioFormat& requestedFormat, uint32_t bufferSizeFrames) = 0;
        virtual void Shutdown() = 0;
        virtual bool Start() = 0;
        virtual bool Stop() = 0;
        virtual bool IsRunning() const = 0;

        virtual AudioDeviceInfo GetInfo() const = 0;
        virtual AudioFormat GetActualFormat() const = 0;
        virtual uint32_t GetActualBufferSizeFrames() const = 0;
        virtual double GetLatencyMilliseconds() const = 0;


        template <typename T>
        T* AsPlatformSpecific() { return dynamic_cast<T>(*this); }


    };
}
