#pragma once

#include <Event.h>

namespace ReMixer {
    class Stream;

    /// TODO: Equalizer Effect
    /// TODO: Compressor Effect
    /// TODO: Reverb Effect
    /// TODO: Chorus Effect
    /// TODO: Distortion Effect
    /// TODO: Echo Effect
    /// TODO: Flange Effect
    /// TODO: Pitch Shift Effect
    /// TODO: Tremolo Effect
    /// TODO: Positional Sound Object?

    /// Physical representation of Pulse-Code-Modulated sound data.
    class PCM { };

    class Decibels { };

    enum class SoundApi
    {
        PulseAudio, /// Currently supported.
        ALSA,       /// Not yet supported.
        PipeWire,   /// Not yet supported.
        WASAPI,     /// Work-in-progress support.
        OSS,        /// Not yet supported.
        CoreAudio   /// Support not planned.
    };


    class AudioDevice {};
    class PlaybackDevice : public AudioDevice {};
    class RecordingDevice : public AudioDevice {};

    /// An abstract class that defines the SoundSubsystem which will be used by the higher-level ReMixer API.
    /// Since it is common for sound APIs to operate on a separate (or even multiple) threads,
    /// The subsystem also functions as a "controller" that manages these separate threads.
    /// Therefore, it is safe to use in a single-threaded application.
    class SoundSubsystem
    {
    public:
        SoundSubsystem() {}

        std::vector<AudioDevice> GetAudioDevices();
        std::vector<PlaybackDevice> GetPlaybackDevices();
        std::vector<RecordingDevice> GetRecordingDevices();

        std::string GetPlaybackDevice();
        std::string GetRecordingDevice();
        void SetPlaybackDevice();
        void SetRecordingDevice();

        void SetMasterVolume(Decibels db);

    protected:
    private:


    };


    /// A Windows Sound API implementation of the SoundSubsystem.
    class WASAPISubsystem : public SoundSubsystem
    {
    public:
    protected:
    private:
    };


}
